/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/std/def.hpp>

#include <keel/typeconv.hpp>

#include <nostalgia/gfx/context.hpp>
#include <nostalgia/gfx/palette.hpp>
#include <nostalgia/gfx/tilesheet.hpp>

namespace nostalgia::gfx {

// Type converters

class NostalgiaPaletteToPaletteV1Converter: public keel::Converter<NostalgiaPalette, PaletteV1> {
	ox::Error convert(keel::Context&, NostalgiaPalette &src, PaletteV1 &dst) const noexcept final;
};

class PaletteV1ToPaletteV2Converter: public keel::Converter<PaletteV1, PaletteV2> {
	ox::Error convert(keel::Context&, PaletteV1 &src, PaletteV2 &dst) const noexcept final;
};

class PaletteV2ToPaletteV3Converter: public keel::Converter<PaletteV2, PaletteV3> {
	ox::Error convert(keel::Context&, PaletteV2 &src, PaletteV3 &dst) const noexcept final;
};

class PaletteV3ToPaletteV4Converter: public keel::Converter<PaletteV3, PaletteV4> {
	ox::Error convert(keel::Context&, PaletteV3 &src, PaletteV4 &dst) const noexcept final;
};

class PaletteV4ToPaletteV5Converter: public keel::Converter<PaletteV4, PaletteV5> {
	ox::Error convert(keel::Context&, PaletteV4 &src, PaletteV5 &dst) const noexcept final;
};

class PaletteToCompactPaletteConverter: public keel::Converter<Palette, CompactPalette> {
	ox::Error convert(keel::Context&, Palette &src, CompactPalette &dst) const noexcept final;
};

class TileSheetV1ToTileSheetV2Converter: public keel::Converter<TileSheetV1, TileSheetV2> {
	ox::Error convert(keel::Context&, TileSheetV1 &src, TileSheetV2 &dst) const noexcept final;
};

class TileSheetV2ToTileSheetV3Converter: public keel::Converter<TileSheetV2, TileSheetV3> {
	static void convertSubsheet(
			TileSheetV2::SubSheet &src,
			TileSheetV3::SubSheet &dst,
			SubSheetId &idIt) noexcept;
	ox::Error convert(keel::Context&, TileSheetV2 &src, TileSheetV3 &dst) const noexcept final;
};

class TileSheetV3ToTileSheetV4Converter: public keel::Converter<TileSheetV3, TileSheetV4> {
	static void convertSubsheet(
			TileSheetV3::SubSheet &src,
			TileSheetV4::SubSheet &dst,
			SubSheetId &idIt) noexcept;
	ox::Error convert(keel::Context&, TileSheetV3 &src, TileSheetV4 &dst) const noexcept final;
};

class TileSheetV4ToTileSheetV5Converter final: public keel::Converter<TileSheetV4, TileSheetV5> {
	static void convertSubsheet(
			int bpp,
			TileSheetV4::SubSheet &src,
			TileSheetV5::SubSheet &dst) noexcept;
	ox::Error convert(keel::Context&, TileSheetV4 &src, TileSheetV5 &dst) const noexcept override;
};

class TileSheetToCompactTileSheetConverter: public keel::Converter<TileSheet, CompactTileSheet> {
	ox::Error convert(keel::Context&, TileSheet &src, CompactTileSheet &dst) const noexcept final;
};

}
