/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <nostalgia/gfx/gfx.hpp>

#include <nostalgia/scene/scene.hpp>

namespace nostalgia::scene {

Scene::Scene(SceneStatic const&sceneStatic) noexcept:
	m_sceneStatic(sceneStatic) {
}

ox::Error Scene::setupDisplay(gfx::Context &ctx) const noexcept {
	if (m_sceneStatic.palettes.empty()) {
		return ox::Error(1, "Scene has no palettes");
	}
	auto const&palette = m_sceneStatic.palettes[0];
	OX_RETURN_ERROR(gfx::loadBgTileSheet(ctx, 0, m_sceneStatic.tilesheet));
	OX_RETURN_ERROR(gfx::loadBgPalette(ctx, 0, palette));
	// disable all backgrounds
	gfx::setBgStatus(ctx, 0);
	for (auto layerNo = 0u; auto const&layer : m_sceneStatic.tileMapIdx) {
		setupLayer(ctx, layer, layerNo);
		++layerNo;
	}
	return {};
}

void Scene::setupLayer(
		gfx::Context &ctx,
		ox::Vector<uint16_t> const&layer,
		unsigned layerNo) const noexcept {
	gfx::setBgStatus(ctx, layerNo, true);
	gfx::setBgCbb(ctx, layerNo, 0);
	auto x = 0;
	auto y = 0;
	const auto width = m_sceneStatic.rows[layerNo];
	for (auto const&tile : layer) {
		const auto tile8 = static_cast<uint8_t>(tile);
		gfx::setBgTile(ctx, layerNo, x, y, tile8);
		gfx::setBgTile(ctx, layerNo, x + 1, y, tile8 + 1);
		gfx::setBgTile(ctx, layerNo, x, y + 1, tile8 + 2);
		gfx::setBgTile(ctx, layerNo, x + 1, y + 1, tile8 + 3);
		x += 2;
		if (x >= width * 2) {
			x = 0;
			y += 2;
		}
	}
}

}
